/*******************************************************************************	
*									       *	
*     Program: Top++ (ver. 1.2)   		                               *
*                                                                              *
*     Autors: Michal Czakon (Aahen) and Alexander Mitov (CERN)                 *
*     Preprint number: CERN-PH-TH/2011-303                                     *
*                                                                              *
*     Licence: GNU Public License. No warranty given or implied.               *
*     Website: http://www.alexandermitov.com/software                          *
*     Compiler: Developed and tested with GCC’s C++ compiler.                  *
*     Operating system: Linux; could be adapted for Mac OS X.                  *
*     Program Language: C++                                                    *
*     External libraries: 1) GNU Scientific Library (GSL)                      *
*                         2) the Les Houches Accord PDF Interface (LHAPDF)     *
*                                                                              *
*     What is this program for?     			                       *
*           		                                                       *	
*     This program calculates the top-pair total inclusive cross-section in    *
*     hadron collisions. It has built-in the most advanced theoretical results *
*     available in the literature. The program is able to calculate the total  *
*     top-pair cross-section in both pure fixed order perturbation theory      *
*     and by including soft-gluon resummation through                          *
*     next-to-next-to-leading logarithmic order (NNLL).                        *
*                                                                              *
*     The underlying physics is described in the paper:                        *
*     M. Cacciari, M. Czakon, M. L. Mangano, A. Mitov and P. Nason,            *
*     arXiv:1111.5869 [hep-ph].                                                *
*     The NNLO result for qq->tt+X implemented here is computed in the paper:  *
*     P. Baernreuther, M. Czakon and A. Mitov, arXiv:1204.5201 [hep-ph].       *
*                                                                              *
*******************************************************************************/


#include <map>
#include <fstream>
#include <sstream>
#include <algorithm>
#include <time.h>
#include <complex>
#include "LHAPDF/LHAPDF.h"
#include <gsl/gsl_errno.h>

#include "PartonicFlux.h"
#include "SubtrFlux.h"
#include "FixedOrder.h"
#include "Resummation.h"
#include "Utilities.h"
#include "psin.h"
#include "lgamma.h"

using std::cout;
using std::endl;
using std::string;
using std::vector;
using std::map;
using std::domain_error;
using std::max;
using std::ofstream;
using std::ifstream;
using std::stringstream;


typedef std::complex<double> dcomp;



int main() {	
  
  gsl_set_error_handler_off();//Supress the GSL-default abort when integration problem occurs. Report it instead.
  time_t start,end, calcstart,calcend; 
  double dif,calcdif;		
  
  vector<vector<double> > result_output;// Stores the global result.	
  time (&calcstart);	
  
/*******************************************************************************	
*									       *	
*		     Default and user_provided parameters		       *
*									       *	
*******************************************************************************/		
  
  map<string, vector<string> > options;
  const int NumberParameters = 30;//Number of param's !!!!!!!!! Must be modified if new options are added !!!!!!!
  
  // defaults for known options. defaults have only one element,
  // whereas the user options allow for lists (e.g. muR)
  
  string defaults[NumberParameters*2] = {
    //================================= General setup (Collider, pdf, F.O. vs RES.)
    "Collider", "TEV",
    "WithResummation", "YES",
    "PDFuncertainty", "NO",
    "RestrictedScaleVariation", "2.0",  
    "PDFset", "MSTW2008nnlo68cl",
    "PDFmember", "0",
    //================================= mt(GeV); muF and muR (in units of mt)
    "Mtop", "173.3",
    "MtopLimit", "-1.0",//unattainable value. Do NOT modify!
    "MtopStep", "1",
    "muF", "1.0",
    "muR", "1.0",		
    //================================= Resummation
    "OrderFO", "NNLO",
    "OrderRES", "NNLL",		
    "A", "2.0",
    "RESonORoff", "OFF",
    "TwoLoopCoulombs", "YES",
    "H2gg1", "0.0",//normalization as/(pi)
    "H2gg8", "0.0",
    "H2qq", "84.81",
    //================================= Fixed Order
    "LO", "YES",
    "NLO", "YES",
    "NNLO", "YES",
    "NNLOonORoff", "OFF",
    "Cbargg", "0",//normalization as/(4pi)
    //================================= Setup parameters
    "ECMLHC", "8000",
    "ECMTEV", "1960",
    "Precision", "2",
    "NPdfGrid", "100",
    "ETA", "1e-5",
    "CMP", "2.7"
  };
  
  for (int i = 0; i < NumberParameters; ++i)
    options[defaults[2*i]] = vector<string>(1, defaults[2*i+1]);
  
  // Override the default options by those from the user (external file). 
  // Every option is a string at this point. 
  // Comments start with a "/"
  
  ifstream config("top++.cfg");
  
  while (config)
    {
      string buffer;
      getline(config, buffer);
      if (buffer.empty()) continue;
      
      stringstream line(buffer);
      string key, val;
      vector<string> values;
      line >> key;
      while (line)
	{
	  val.clear();
	  line >> val;
	  if (val.empty()) continue;
	  if (val[0] == '/') break;
	  values.push_back(val);
	}
      
      options[key] = values;
    }
  
  // assignment of the actual options
  
  double A = atof(options["A"][0].c_str());
  double CMP = atof(options["CMP"][0].c_str());
  double Cbargg = atof(options["Cbargg"][0].c_str());
  double ECMLHC = atof(options["ECMLHC"][0].c_str());
  double ECMTEV = atof(options["ECMTEV"][0].c_str());
  double ETA = atof(options["ETA"][0].c_str());
  double H2gg1 = atof(options["H2gg1"][0].c_str());
  double H2gg8 = atof(options["H2gg8"][0].c_str());
  double H2qq = atof(options["H2qq"][0].c_str());
  double Mtop = atof(options["Mtop"][0].c_str());
  double MtopLimit   = atof(options["MtopLimit"][0].c_str());
  double MtopStep = atof(options["MtopStep"][0].c_str());
  int NPdfGrid = atoi(options["NPdfGrid"][0].c_str());
  int PDFmember = atoi(options["PDFmember"][0].c_str());
  int Precision = atoi(options["Precision"][0].c_str());
  
  string LO = options["LO"][0];
  string NLO = options["NLO"][0];
  string NNLO = options["NNLO"][0];
  string NNLOonORoff = options["NNLOonORoff"][0];
  string OrderFO = options["OrderFO"][0];
  string OrderRES = options["OrderRES"][0];
  string PDFuncertainty = options["PDFuncertainty"][0];
  string RESonORoff = options["RESonORoff"][0];
  string WithResummation = options["WithResummation"][0];
  string Collider = options["Collider"][0];
  string PDFset = options["PDFset"][0];
  string TwoLoopCoulombs = options["TwoLoopCoulombs"][0];
  string RestrictedScaleVariation = options["RestrictedScaleVariation"][0];
  
  
  // Check if this option's value is allowed (either "NO" or a "double >= 1.0"):
  // Recall: atof(ptr-to-non-double) == 0.0
  if (RestrictedScaleVariation != "NO" && 
      atof(RestrictedScaleVariation.c_str()) < 1.0) 
    {
      throw domain_error("Incorrect option RestrictedScaleVariation: (must be either NO or a double >= 1.0).");
    }
  
  // Set default value MtopLimit=Mtop.
  if (MtopLimit == -1.0){ MtopLimit = Mtop;}
  
  // Guard the range of this option to avoid infinite loops:
  if (MtopStep <= 0) 
    {
      throw domain_error("Incorrect option MtopStep: (must be a positive number).");
    }
  
  vector<double> vmuF, vmuR;
  if (PDFuncertainty == "YES") 
    {//For pdf variation, use oly central scales.
      vmuF.push_back(1.0);
      vmuR.push_back(1.0);
    }	
  else 
    {
      for (vector<string>::iterator mu = options["muR"].begin();
	   mu != options["muR"].end(); ++mu) vmuR.push_back(atof(mu->c_str()));
      for (vector<string>::iterator mu = options["muF"].begin();
	   mu != options["muF"].end(); ++mu) vmuF.push_back(atof(mu->c_str()));	
      
      if (vmuF.size() == 0) 
	{
	  throw domain_error("Please specify values for muF.");
	}
      if (vmuR.size() == 0) 
	{
	  throw domain_error("Please specify values for muR.");
	}
    }
  
  // When with 'RESUMMATION', set the F.O. parameters according to the RES ones:
  if (WithResummation=="YES") 
    {      
      if (OrderFO=="LO") 
	{	
	  LO =   "YES";
	  NLO =  "NO";
	  NNLO = "NO";
	}
      else if (OrderFO=="NLO") 
	{	
	  LO =   "YES";
	  NLO =  "YES";
	  NNLO = "NO";
	}
      else if (OrderFO=="NNLO") 
	{	
	  LO =   "YES";
	  NLO =  "YES";
	  NNLO = "YES";
	  NNLOonORoff = RESonORoff;// afects only the gg-reaction
	  Cbargg = - 1136.8106084958624 
	    + 4.57142857142857*H2gg1 
	    + 11.428571428571429*H2gg8;
	}
      else 
	{
	  throw domain_error("Unknown option for Resummation matching (Expected LO, NLO or NNLO)");
	}
    }	
  
  double ECM;//Collider c.m. energy
  if (Collider == "TEV") { ECM=ECMTEV; }
  else { ECM=ECMLHC; }
  
  
/*******************************************************************************	
*									       *	
*	          Check for consistency and completeness all string            *
*             options that are passed as bool to the classes.                  *
*									       *	
*******************************************************************************/		
  
  if (Collider!="LHC" && Collider!="TEV")
    {
      throw domain_error("Unknown type of collider (possibilities: TEV or LHC).");
    }
  
  if (LO!="YES" && LO!="NO")
    {
      throw domain_error("Unknown option for the LO result (possibilities: YES or NO).");
    }
  
  if (NLO!="YES" && NLO!="NO")
    {
      throw domain_error("Unknown option for the NLO result (possibilities: YES or NO).");
    }
  
  if (NNLO!="YES" && NNLO!="NO")
    {
      throw domain_error("Unknown option for the NNLO result (possibilities: YES or NO).");
    }
      
  if (NNLOonORoff!="ON" && NNLOonORoff!="OFF")
    {
      throw std::domain_error("Unknown option for the NNLO_approx result (possibilities: ON or OFF).");
    }
  
  if (RESonORoff!="ON" && RESonORoff!="OFF")
    {	
      throw domain_error("Unknown option with resummation at NNLL (possibilities: OFF or ON).");
    }
  
  if (TwoLoopCoulombs!="YES" && TwoLoopCoulombs!="NO")
    {	
      throw domain_error("Unknown option about Two-loop Coulombs (possibilities are: YES or NO).");
    }
  
  if (OrderRES!="LL" && OrderRES!="NLL" && OrderRES!="NNLL")
    {	
      throw domain_error("Unknown option with resummation order (possibilities: LL, NLL, NNLL).");
    }
  
  if (OrderFO!="LO" && OrderFO!="NLO" && OrderFO!="NNLO")
    {	
      throw domain_error("Unknown option with resummation matching to F.O. (possibilities: LO, NLO, NNLO).");
    }
  
  
/*******************************************************************************	
*									       *	
*			     Initialize the PDF set			       *
*									       *	
*******************************************************************************/	
  
  time (&start);
  initPDFSetByName(PDFset,LHAPDF::LHGRID);
  time (&end);
  dif = difftime (end,start);
  cout << "................. PDF initialized (it took " 
       << dif 
       << " [sec.]).\n" 
       << endl;
  
  
/*******************************************************************************	
*									       *	
*		       BEGIN: loop over the top mass mt			       *
*									       *	
*******************************************************************************/		
  
  //check user's input for consistency	
  if (MtopLimit < Mtop) 
    {
      MtopLimit = Mtop;
      cout << "\t!!! Warning !!! : MtopLimit is smaller than Mtop. "
	"The program will continue with MtopLimit=Mtop.\n" 
	   << endl;
    }	
  
  for (double mt=Mtop; mt <=MtopLimit; mt+=MtopStep) {
    double Rhad = 4*pow(mt/ECM,2);
    
    
/*******************************************************************************	
*									       *	
*                       Initialize the partonic fluxes			       *
*									       *	
*******************************************************************************/	
    
    time (&start);
    vector<PartonicFlux> vFlux;
    
    if (PDFuncertainty == "YES") //all pdf members; a single muF value (=mt)
      {
	cout << "Initializing the fluxes for each pdf member ... " 
	     << endl;	
	for(int i = 0; i <= LHAPDF::numberPDF(); ++i) 
	  {
	    PartonicFlux flux(Collider=="LHC" ? 1:0 , i , Rhad, 
			      1.0, vmuF[0] * mt, NPdfGrid, Precision);
	    vFlux.push_back( flux );
	    cout << "\t(" 
		 << i 
		 << " out of "
		 << 1+LHAPDF::numberPDF() 
		 << ")" 
		 << endl;
	  }
      }
    else // all user's muF values; A single pdf member
      {
	cout << "Initializing the fluxes for pdf member " 
	     << PDFmember 
	     <<" ..." 
	     << endl;	
	for(vector<double>::iterator it = vmuF.begin(); it != vmuF.end(); ++it) 
	  {
	    PartonicFlux flux(Collider=="LHC" ? 1:0, 
			      PDFmember, 
			      Rhad, 
			      1.0, 
			      (*it)*mt, 
			      NPdfGrid, 
			      Precision);
	    vFlux.push_back( flux );
	  }
      }
    time (&end);
    dif = difftime (end,start);
    cout << ".............. Fluxes initialized (it took " 
	 << dif 
	 << " [sec.]).\n" 
	 << endl;
    
    
/*******************************************************************************	
*									       *	
*		       	Initialize the FixedOrder result		       *
*									       *	
*******************************************************************************/
    
    time (&start);
    cout << "\nInitializing Fixed Order ..." 
	 << endl;
    vector<FixedOrder> vFO;
    
    for(vector<double>::iterator itF = vmuF.begin(); itF != vmuF.end(); ++itF) 
      {
	for(vector<double>::iterator itR = vmuR.begin(); itR != vmuR.end(); ++itR) 
	  {
	    FixedOrder FO(mt, 
			  (*itF)*mt, 
			  (*itR)*mt, 
			  LO=="YES" ? 1:0 , 
			  NLO=="YES" ? 1:0 , 
			  NNLO=="YES" ? 1:0 ,  
			  NNLOonORoff=="ON" ? 1:0 , 
			  Cbargg);
	    vFO.push_back( FO );
	  }
      }	
    time (&end);
    dif = difftime (end,start);
    cout << "......... Fixed Order initialized (it took " 
	 << dif 
	 << " [sec.]).\n" 
	 << endl;
    
    
/*******************************************************************************	
*      					                                       *	
*                        Initialize the Subtraction flux 		       *
*								               *	
*******************************************************************************/
    
    vector<SubtrFlux> vSF;
    if (WithResummation == "YES") 
      {					
	time (&start);
	cout << "Initializing the subtraction fluxes ... " 
	     << endl;	
	
	for(vector<PartonicFlux>::iterator it = vFlux.begin(); it != vFlux.end(); ++it) 
	  {
	    SubtrFlux SF(*it , Rhad, ETA);
	    vSF.push_back( SF );
	  }
	time (&end);
	dif = difftime (end,start);
	cout << "...... Subtraction Fluxes initialized (it took "
	     << dif 
	     << " [sec.]).\n" 
	     << endl;
      }
    
    
/*******************************************************************************	
*					 			               *	
*			      Initialize Resummation			       *
*								       	       *	
*******************************************************************************/
    
    vector<Resummation> vRES;
    if (WithResummation == "YES") 
      {		
	time (&start);
	cout << "Initializing resummation ... " 
	     << endl;
	if (PDFuncertainty == "YES") 
	  {
	    for(int i = 0; i <= LHAPDF::numberPDF(); ++i) 
	      {	
		Resummation RES(vSF[i], 
				Precision, 
				mt, 
				vmuF[0]*mt, 
				vmuR[0]*mt, 
				A, 
				CMP, 
				IntegerForm(OrderFO), 
				IntegerForm(OrderRES), 
				RESonORoff=="OFF" ? 1:0 , 
				TwoLoopCoulombs=="YES" ? 1:0 , 
				H2qq,H2gg1,H2gg8);
		vRES.push_back( RES );
	      }
	  }
	else 
	  {
	    for(int i=0; i != vmuF.size(); ++i) 
	      {
		for(int j=0; j != vmuR.size(); ++j) 
		  {	
		    Resummation RES(vSF[i], 
				    Precision, 
				    mt, 
				    vmuF[i]*mt, 
				    vmuR[j]*mt, 
				    A, 
				    CMP, 
				    IntegerForm(OrderFO),
				    IntegerForm(OrderRES), 
				    RESonORoff=="OFF" ? 1:0 , 
				    TwoLoopCoulombs=="YES" ? 1:0 , 
				    H2qq,H2gg1,H2gg8);
		    vRES.push_back( RES );
		  }
	      }
	  }
	time (&end);
	dif = difftime (end,start);
	cout << "...... Resummation initialization done (it took " 
	     << dif 
	     << " [sec.]).\n" 
	     << endl;
      }
    
/*******************************************************************************	
*		 			                                       *	
*			  Compute the final result			       *
*								               *	
*******************************************************************************/

    time (&start);
    cout << "\nComputing the cross-section:\n" 
	 << endl;
    
    double central = -1.0;//an unattainable value for sigma_tot
    vector<double> result;
    
    if (PDFuncertainty == "YES") //compute sigma for all pdf members, and for a single (muF,muR) value 
      {
	if (vFO.size() != 1 || vmuF.size() != 1 || vmuR.size() != 1) 
	  {
	    throw domain_error("Inconsistent number of (muF,muR)"
			       " pairs with PDF uncertainty determination."
			       " (should be only one)" );
	  }
	bool pass = ScalesPass(RestrictedScaleVariation,vmuF[0],vmuR[0]);
	for(int i = 0; i <= LHAPDF::numberPDF(); ++i) 
	  {
	    if (pass)
	      {
		if (WithResummation == "YES") 
		  {			
		    result.push_back(  ComputeFinalResultFO (vFlux[i],vFO[0],Rhad,Precision)
				       + ComputeFinalResultRES(vFlux[i], vSF[i], vFO[0], vRES[i], Rhad, Precision) );
		  }
		else 
		  {
		    result.push_back(  ComputeFinalResultFO (vFlux[i],vFO[0],Rhad,Precision) );
		  }
		if (i == 0) 
		  {
		    central = result.back();
		  }
		cout << " Pdf member " 
		     << i 
		     << ", sigma_tot(mtop="
		     << mt 
		     << ",muF="
		     << vmuF[0]
		     << ",muR="
		     << vmuR[0]
		     << ") = " 
		     << result.back()
		     << " [pb]." 	
		     << endl;
	      }
	  }
	time (&end);
	dif = difftime (end,start);
	
	if (result.empty()) 
	  {
	    throw domain_error("The pair (muF,muR) does not meet the restricted scale variation criteria."
			       " Please review muF and muR.");
	  }
	else 
	  {
	    cout << "\n......... Cross-section computed. (it took " 
		 << dif 
		 << " [sec.]).\n" 
		 << endl;
	  }
      }	
    else //compute sigma for the central pdf member, and for all user supplied (muF,muR) values
      {
	for(int i = 0; i != vmuF.size(); ++i) 
	  {
	    for(int j = 0; j != vmuR.size(); ++j) 
	      {
		bool pass = ScalesPass(RestrictedScaleVariation,vmuF[i],vmuR[j]);
		if (pass)
		  {
		    if (WithResummation == "YES") 
		      {				
			result.push_back(  ComputeFinalResultFO (vFlux[i],vFO[ i * vmuR.size() + j ],
								 Rhad,Precision)
					   + ComputeFinalResultRES(vFlux[i], vSF[i], vFO[ i * vmuR.size() + j ], 
								   vRES[ i * vmuR.size() + j ], Rhad, Precision) );
		      }
		    else 
		      {
			result.push_back(  ComputeFinalResultFO (vFlux[i],vFO[ i * vmuR.size() + j ],Rhad,Precision) );  
		      }
		    if (vmuF[i] == 1.0 && vmuR[j] == 1.0) 
		      {
			central = result.back();
		      }
		    cout << " mtop="
			 << mt
			 << ", muF="
			 << vmuF[i]
			 << ", muR="
			 << vmuR[j]
			 << ", sigma_t(muF,muR) = " 
			 << result.back() 
			 << " [pb]." 
			 << endl;
		  }
	      }
	  }
	time (&end);
	dif = difftime (end,start);
	
	if (result.empty()) 
	  {
	    throw domain_error("Not a single pair (muF,muR) meets the restricted scale-variation criteria."
			       " Please review your muF and muR or the option RestrictedScaleVariation.");
	  }
	else 
	  {
	    cout << "\n......... Cross-section computed! (it took " 
		 << dif 
		 << " [sec.]).\n" 
		 << endl;
	  }
      }
    
    
/*******************************************************************************	
*									       *	
*     Compute scale/pdf variation for a fixed mtop. Output on the screen.      *
*									       *	
*******************************************************************************/	
    
    double s0 = central;
    double smax=-1.0, smin=-1.0;//unattainable values
    
    if (PDFuncertainty == "YES")
      {
	if (PDFset.substr(0,5) == "NNPDF")
	  {
	    deltaPDFnnpdf(result, smax, smin);
	  }
	else 
	  {
	    deltaPDFasymmetric(result, central, smax, smin);
	  }
      }
    else
      {
	smax = *max_element(result.begin(),result.end());
	smin = *min_element(result.begin(),result.end());
      }
    
    
    //Output the result	(for fixed mt)
    if (s0 == -1.0) 
      {
	cout << "\nWARNING: Central value cannot be determined."
	  " Please include the point muF=muR=mt.\n" 
	     << endl;
      }
    else 
      {
	if (PDFuncertainty == "YES") 
	  {
	    cout << "****************    Final result (PDF uncertainty):   ****************" 
		 << endl;		
	  }
	else 
	  {
	    cout << "****************    Final result (scale variation):   ****************" 
		 << endl;
	  }
	cout << " " << endl;
	
	cout << " sigma_tot = " 
	     << central 
	     << " + " 
	     << smax - s0 
	     << " (" 
	     << (smax - s0)/s0*100.0 
	     << " \%) - " 
	     << s0 - smin 
	     << " (" 
	     << (s0 - smin)/s0*100.0 
	     << " \%)"
	     << " [pb]." 
	     << endl;
	
	cout << " " << endl;
	cout << "**********************************************************************" 
	     << endl;
      }
    cout << "\n" << endl;
    
    vector<double> vtemp;// the result for fixed m_top
    vtemp.push_back(mt); 
    vtemp.push_back(s0); 
    vtemp.push_back(smin); 
    vtemp.push_back(smax); 
    
    result_output.push_back(vtemp);//the result for all values of m_top	
    
  }//END loop over top mass mt.
  
  time (&calcend);
  calcdif = difftime (calcend,calcstart);
  
  
/*******************************************************************************	
*									       *	
*		  Print the complete output (gnuplot format):		       *
*									       *	
*******************************************************************************/
  
  ofstream file("top++.res");
  
  cout << "!!!!!!!!!!!!!!!! A summary of the complete calculation: !!!!!!!!!!!!!!!!\n" 
       << endl;	
  cout << "Total time used: " 
       << calcdif 
       << " [sec.].\n" 
       << endl;
  
  if (PDFuncertainty == "YES")
    {
      cout << "#mt central   pdf_min   pdf_max"
	   << endl;	
      
      file << "# PDF variation for sigma_tot. m_top in the range (" 
	   << Mtop 
	   << "," 
	   << MtopLimit 
	   << ")." 
	   << endl;
      file << "# Collider " 
	   << Collider 
	   << " with energy " 
	   << ECM << " GeV" 
	   << endl;
      file << "# PDF set " 
	   << PDFset 
	   << endl;
      if (WithResummation == "YES")
	{
	  file << "# Resummed calculation: " 
	       << OrderFO 
	       << " + " 
	       << OrderRES  
	       << ", A = " 
	       << A 
	       << endl;
	}
      else 
	{
	  file << "# Fixed Order calculation with options: "
	       << endl;
	  file << "# LO(" 
	       << LO 
	       << "), NLO(" 
	       << NLO 
	       << "), NNLO(" 
	       << NNLO 
	       <<")" 
	       << ", 2-loop scale terms: " 
	       << NNLOonORoff 
	       << "\n" 
	       << endl;	
	}
      file << "#mt central   pdf_min   pdf_max"
	   << endl;	
    }	
  else 
    {
      cout << "#mt central scale_min scale_max"
	   << endl;	
      
      if (RestrictedScaleVariation == "NO")
	{
	  file << "# Unrestricted scale variation for sigma_tot. m_top in the range (" 
	       << Mtop 
	       << "," 
	       << MtopLimit 
	       << ")." 
	       << endl;
	}
      else
	{
	  file << "# Restricted scale variation for sigma_tot with "
	       << "RestrictedScaleVariation = " 
	       << RestrictedScaleVariation << endl;
	  file << "# m_top in the range (" 
	       << Mtop 
	       << "," 
	       << MtopLimit 
	       << ")." 
	       << endl;
	}
      
      file << "# Collider " 
	   << Collider 
	   << " with energy " 
	   << ECM 
	   << " GeV" 
	   << endl;
      file << "# PDF set " 
	   << PDFset 
	   << endl;
      if (WithResummation == "YES")
	{
	  file << "# Resummed calculation: " 
	       << OrderFO 
	       << " + " 
	       << OrderRES 
	       << ", A = " 
	       << A 
	       << endl;
	}
      else 
	{
	  file << "# Fixed Order calculation with options: "
	       << endl;
	  file << "# LO(" 
	       << LO 
	       << "), NLO(" 
	       << NLO 
	       << "), NNLO(" 
	       << NNLO 
	       <<")" 
	       << ", 2-loop scale terms: " 
	       << NNLOonORoff 
	       << endl;	
	}
      file << "#mt central scale_min scale_max"
	   << endl;	
    }
  
  for(vector<vector<double> >::iterator it = result_output.begin(); 
      it != result_output.end(); 
      ++it) 
    {	
      for(vector<double>::iterator j = (*it).begin(); 
	  j != (*it).end(); 
	  ++j) 
	{	 	
	  cout <<  *j << "  ";
	  file <<  *j << "  ";
	}
      cout << endl;
      file << endl;
    }
  cout << "\n" << endl;
  
  return 0;
}
